#!/bin/sh
set -e
NAME="$(basename "$0")"
cd "$(dirname "$0")"

CONF='Release'
MAKEDOC='no'
RUNTEST='no'
CC='cc'
MPICC='mpicc'
MSBUILD='msbuild'
NUGET='nuget'

eval set -- "$(getopt -o c:dt --long configuration:,document,test,cc:,mpicc:,msbuild:,nuget: -n "$NAME" -- "$@")"
while true; do
    case "$1" in
    -c|--configuration)
        CONF="$2"
        shift 2
        ;;
    -d|--document)
        MAKEDOC='yes'
        shift 1
        ;;
    -t|--test)
        RUNTEST='yes'
        shift 1
        ;;
    --cc)
        CC="$2"
        shift 2
        ;;
    --mpicc)
        MPICC="$2"
        shift 2
        ;;
    --msbuild)
        MSBUILD="$2"
        shift 2
        ;;
    --nuget)
        NUGET="$2"
        shift 2
        ;;
    --)
        shift
        break
        ;;
    *)
        echo "$NAME: Error processing command line options" >&2
        exit 1
        ;;
    esac
done

ARCH="$(uname -sm | tr 'A-Z ' 'a-z-')"

export LD_LIBRARY_PATH="$PWD/LPSolve/bin/$CONF/$ARCH:$PWD/MPIGlue/bin/$CONF/$ARCH:$LD_LIBRARY_PATH"
export DYLD_LIBRARY_PATH="$PWD/LPSolve/bin/$CONF/$ARCH:$PWD/MPIGlue/bin/$CONF/$ARCH:$DYLD_LIBRARY_PATH"

$MSBUILD /property:Configuration="$CONF" /target:Restore EoS.sln
$MSBUILD /property:Configuration="$CONF" /target:Build EoS.sln

make -C LPSolve CC="$CC" CONF="$CONF" ARCH="$ARCH"
make -C MPIGlue CC="$MPICC" CONF="$CONF" ARCH="$ARCH"

if [ "$MAKEDOC" != "no" ]; then
    mkdir -p "doc/$CONF/"
    mono "EoS.DocumentationTool/bin/$CONF/net45/eosdoc.exe" -o "doc/$CONF/" \
    "EoS.Core/bin/$CONF/net45/EoS.Core.dll" \
    "EoS.DebyeModel/bin/$CONF/net45/EoS.DebyeModel.dll" \
    "EoS.PolynomialModel/bin/$CONF/net45/EoS.PolynomialModel.dll" \
    "EoS.Optimization/bin/$CONF/net45/EoS.Optimization.dll"
fi

if [ "$RUNTEST" != "no" ]; then
    if [ ! -d NUnit.ConsoleRunner ]; then
        $NUGET install NUnit.ConsoleRunner -Version 3.8.0 -ExcludeVersion
        $NUGET install NUnit.Extension.NUnitProjectLoader -Version 3.6.0 -ExcludeVersion
    fi
    mono "NUnit.ConsoleRunner/tools/nunit3-console.exe" -config="$CONF" EoS.Test.nunit
fi
