#!/bin/sh
set -e
NAME="$(basename "$0")"
cd "$(dirname "$0")"

CSC='mcs'
CONF='Release'
INSTALL='cp -fp'
STUBEXE='no'
STUBPSH='yes'
MAKEZIP='no'

eval set -- "$(getopt -o c:sz --long configuration:,csc:,symlink,install:,stub-exe::,stub-shell::,zip -n "$NAME" -- "$@")"
while true; do
    case "$1" in
        -c|--configuration)
            CONF="$2"
            shift 2
            ;;
        --csc)
            CSC="$2"
            shift 2
            ;;
        -s|--symlink)
            INSTALL='ln -fsr'
            shift
            ;;
        --install)
            INSTALL="$2"
            shift 2
            ;;
        --stub-exe)
            STUBEXE="${2:-yes}"
            shift 2
            ;;
        --stub-shell)
            STUBPSH="${2:-yes}"
            shift 2
            ;;
        -z|--zip)
            MAKEZIP='yes'
            shift
            ;;
        --)
            shift
            break
            ;;
        *)
            echo "$NAME: Error processing command line options" >&2
            exit 1
            ;;
    esac
done

if [ -n "$1" ]; then
    STAGE="$1"
else
    STAGE="EoS-$(monop -r EoS.Core/bin/$CONF/net45/EoS.Core.dll | awk '/^Version=/{print(substr($0,index($0,"=")+1))}')"
fi

ARCH="$(uname -sm | tr 'A-Z ' 'a-z-')"
NEUTLIB="$STAGE/lib/eos"
ARCHLIB="$STAGE/lib/eos/$ARCH"

mkdir -p "$NEUTLIB"
mkdir -p "$ARCHLIB"

$INSTALL "LICENSE.txt" "$STAGE/"
$INSTALL "README.md" "$STAGE/"
$INSTALL "ICON.svg" "$STAGE/"
if [ "$STUBPSH" != 'no' ]; then
    $INSTALL "Launcher.sh" "$STAGE/eos"
fi
if [ "$STUBEXE" != 'no' ]; then
    $CSC -optimize+ -target:exe -win32icon:"ICON.ico" "Launcher.cs" -out:"$STAGE/eos.exe"
fi
for it in LPSolve MPIGlue; do
    $INSTALL "$it/bin/$CONF/$ARCH"/* "$ARCHLIB/"
done
$INSTALL "EoS.Core/bin/$CONF/net45/System.ValueTuple.dll" "$NEUTLIB/"
$INSTALL "EoS.Core/bin/$CONF/net45/FSharp.Core.dll" "$NEUTLIB/"
for it in EoS.Core EoS.Optimization EoS.DebyeModel EoS.PolynomialModel EoS.CommandLine; do
    $INSTALL "$it/bin/$CONF/net45/$it.dll" "$NEUTLIB/"
    $INSTALL "$it/bin/$CONF/net45/$it.pdb" "$NEUTLIB/"
    $INSTALL "$it/bin/$CONF/net45"/*.xml "$NEUTLIB/"
done
for it in EoS.FormulaTool EoS.PropertyTool EoS.FittingTool EoS.OptimizationTool EoS.IndexTool EoS.AdiabatTool EoS.BitmapViewer; do
    $INSTALL "$it/bin/$CONF/net45"/*.exe "$NEUTLIB/"
    $INSTALL "$it/bin/$CONF/net45"/*.pdb "$NEUTLIB/"
done

if [ "$MAKEZIP" != 'no' ]; then
    zip -ru "$STAGE.zip" "$STAGE"
fi
