// This file is part of EoS
// Copyright (c) 2009-2016 Thomas Chust
//                         Bayerisches Geoinstitut, Bayreuth
//                         Ludwig-Maximilians-Universität, München
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Reflection;

public static class Launcher {
  static readonly char[] EscapeMe = {' ', '"'};

  public static int Main(string[] args) {
    var root =
      Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
    if (Path.GetFileName(root) == "bin") {
      root = Path.GetDirectoryName(root);
    }

    var lib = Path.Combine(root, "lib", "eos");

    if (args.Length < 1) {
      Console.WriteLine("Available tools:");
      foreach (var tool in Directory.EnumerateFiles(lib, "eos*.exe"))
	Console.WriteLine(Path.GetFileNameWithoutExtension(tool).Substring(3));
      return 0;
    }

    var arch =
      Environment.OSVersion.Platform.ToString().ToLowerInvariant() +
      (Environment.Is64BitProcess ? "_64" : "");
    
    var argstr = new StringBuilder();
    for (var i = 1; i < args.Length; i++) {
      if (i > 1)
	argstr.Append(' ');
      if (args[i].IndexOfAny(EscapeMe) >= 0) {
	argstr.Append('"');
	foreach (var chr in args[i]) {
	  argstr.Append(chr);
	  if (chr == '"') argstr.Append(chr);
	}
	argstr.Append('"');
      }
      else
	argstr.Append(args[i]);
    }

    var psi = new ProcessStartInfo(
      Path.Combine(lib, "eos" + args[0] + ".exe"),
      argstr.ToString());
    psi.EnvironmentVariables["PATH"] =
      Path.Combine(lib, arch) + Path.PathSeparator + psi.EnvironmentVariables["PATH"];
    psi.UseShellExecute = false;

    try {
      var child = Process.Start(psi);
      child.WaitForExit();
      return child.ExitCode;
    }
    catch (Exception exn) {
      Console.WriteLine(exn);
      return 1;
    }
  }
}
